/* dial -- dial phone numbers using a modem.
   Copyright (C) 1997 Sverre Hvammen Johansen

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* AIX requires this to be the first thing in the file.  */
#if defined (_AIX) && !defined (__GNUC__)
 #pragma alloca
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <errno.h>
#include "getopt.h"

#ifndef errno
extern int errno;
#endif

#ifdef	STDC_HEADERS
#include <stdlib.h>
#else	/* Not STDC_HEADERS */
extern void exit ();
extern char *malloc ();
#endif	/* STDC_HEADERS */

#ifdef	HAVE_STRINGS_H
#include <strings.h>
#else
#include <string.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif

#ifdef	__GNUC__
#undef	alloca
#define	alloca(n)	__builtin_alloca (n)
#else	/* Not GCC.  */
#ifdef	HAVE_ALLOCA_H
#include <alloca.h>
#else	/* Not HAVE_ALLOCA_H.  */
#ifndef	_AIX
extern char *alloca ();
#endif	/* Not AIX.  */
#endif	/* HAVE_ALLOCA_H.  */
#endif	/* GCC.  */

struct option longopts[] =
  {
    { "version", 0, 0, 'v' },
    { "help", 0, 0, 'h' },
    { 0, 0, 0, 0 }
  };

extern char version[];

char usage[] = "Usage: %s [-hv] [--help] [--version]\n";

static char *progname;

int
main (argc, argv)
     int argc;
     char *argv[];
{
  int optc;
  int h = 0, v = 0, lose = 0;

  progname = argv[0];

  while ((optc = getopt_long (argc, argv, "hv", longopts, (int *) 0)) != EOF)
    {
      switch (optc)
	{
	case 'v':
	  v = 1;
	  break;
	case 'h':
	  h = 1;
	  break;
	default:
	  lose = 1;
	  break;
	}
    }

  if (lose || optind < argc)
    {
      /* Print error message and exit.  */
      fprintf (stderr, usage, progname);
      exit (1);
    }

  if (v)
    {
      /* Print version number.  */
      fprintf (stderr, "%s\n", version);
      if (! h)
	exit (0);
    }

  if (h)
    {
      /* Print help info and exit.  */
      fputs ("This is Dial, dial phone numbers using a modem.\n",
	     stderr);
      fprintf (stderr, usage, progname);
      fputs ("  -h, --help\t\t\tPrint a summary of the options\n", stderr);
      fputs ("  -v, --version\t\t\tPrint the version number\n", stderr);
      fputs ("Dial is interactive, try the help command for a short overview.\n", stderr);
      exit (0);
    }

  /* Print greeting message and exit.  */

  else
    {
      setup_modem();
      yylex();
      cleanup_modem();
    }

  exit (0);
}

/* Used by alloca.c.  */

char *
xmalloc (size)
     unsigned int size;
{
  char *ptr = malloc (size);
  if (! ptr)
    {
      fprintf (stderr, "%s: virtual memory exhausted\n", progname);
      die (1);
    }
  return ptr;
}
