%{
/* Copyright (C) 1997 Sverre Hvammen Johansen

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#if HAVE_SYSLOG_H
#include <syslog.h>
#endif
%}
%option noyywrap
DIGIT	[0-9abcd#*!,@wtp]
DIGITS	{DIGIT}+
SEPARATOR	"-"|"_"|[ \t]*
%%
help{SEPARATOR}		{system("cat <<EOF |less
An overview of the commands that can be entered:

[0-9ABCD#*!,@WTP]+ directs the modem to dial a phone number.
\t\tThe phone number can contain the numbers 0 through 9 plus
\t\tthe characters A, B, C, D, #, and *.
\t\tThe modem will stay in command mode after dialing so that
\t\tit will not try to connect if it receives a carrier.
\t\tThe phone number may contain the following dial modifiers:
\t\t ! - Hook flash      W - Wait for subsequent dial tone
\t\t , - Pause dial      T - Tone dial
\t\t @ - Wait for quiet  P - Pulse dial
DIAL {number}\tdirect the modem to dial a phone number
TONE {number}\tdirect the modem to tone-dial a phone number
PULSE {number}\tdirect the modem to pulse-dial a phone number
HOOK-FLASH\ttell the modem to go on-hook for a short periode to
\t\taccess phone services, such as transfering a call
PAUSE\t\ttell the modem to wait while dialing
WAIT-FOR-QUIET\ttell the modem to wait for a specified time of silence
WAIT-FOR-TONE\ttell the modem to wait for a subsequent dial tone
SLEEP {SECONDS}\tSleep a specified number of seconds

ON-HOOK\t\thang up the phone
OFF-HOOK\tpick up the phone
SPEAKER-ON\tturn the speaker on
SPEAKER-OFF\tturn the speaker off
LOW-VOLUME\tselect low volume
MEDIUM-VOLUME\tselect medium volume
HIGH-VOLUME\tselect high volume

ENCODE ...\tencode the rest of the line where A=10, B=11, ... =35, 
\t\t.=40, ,=41, ?=42, !=43, \"=44, -=45, &=45, +=47, |=48, 
\t\tand {blank}=55

# ...\t\ttreat the rest of the line as a comment
HELP\t\tproduce this help message
QUIT\t\tclean up and exit
\t\t
EOF");}

dial{SEPARATOR}{DIGITS}	dial(&yytext[4]);
tone{SEPARATOR}{DIGITS}	{yytext[3]='t';dial(&yytext[3]);}
pulse{SEPARATOR}{DIGITS}	{yytext[4]='p';dial(&yytext[4]);}
{DIGITS}		dial(yytext);

hook{SEPARATOR}flash	dial("!");
flash{SEPARATOR}hook	dial("!");
pause			dial(",");
wait{SEPARATOR}for{SEPARATOR}quiet	dial("@");
wait{SEPARATOR}for{SEPARATOR}silence	dial("@");
wait{SEPARATOR}for{SEPARATOR}tone	dial("w");
sleep{SEPARATOR}[1-9][0-9]*	{ int sec; sscanf(&yytext[5],"%d",&sec);sleep(sec);}

on{SEPARATOR}volume	set_speaker_on();
volume{SEPARATOR}on	set_speaker_on();
on{SEPARATOR}speaker	set_speaker_on();
speaker{SEPARATOR}on	set_speaker_on();

off{SEPARATOR}volume	set_speaker_off();
volume{SEPARATOR}off	set_speaker_off();
off{SEPARATOR}speaker	set_speaker_off();
speaker{SEPARATOR}off	set_speaker_off();

low{SEPARATOR}volume	set_low_volume();
volume{SEPARATOR}low	set_low_volume();

medium{SEPARATOR}volume	set_medium_volume();
volume{SEPARATOR}medium	set_medium_volume();

high{SEPARATOR}volume	set_high_volume();
volume{SEPARATOR}high	set_high_volume();

on{SEPARATOR}hook	on_hook();
hook{SEPARATOR}on	on_hook();
hang{SEPARATOR}up	on_hook();

off{SEPARATOR}hook	off_hook();
hook{SEPARATOR}off	off_hook();
pick{SEPARATOR}up	off_hook();

encode.*		encode(&yytext[6]);

quit			return;
exit			return;
^"."			return;

"#".*			/* Comment, ignore */
[ \t\n]			/* WhiteSpace, ignore */
[a-z]+                  fprintf(stderr,"Illegal command (%s)\n",yytext);
.			fprintf(stderr,"Illegal character (%s)\n",yytext);
%%

/*
 * die - like quit, except we can specify an exit status.
 */
void
die(status)
    int status;
{
    syslog(LOG_INFO, "Exit.");
    cleanup_modem();
    exit(status);
}

