/* Copyright (C) 1997 Sverre Hvammen Johansen

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include <stdio.h>
#ifdef	HAVE_STRINGS_H
#include <strings.h>
#else
#include <string.h>
#endif
#include <signal.h>
#include <errno.h>
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#if HAVE_SYSLOG_H
#include <syslog.h>
#endif
#include <netdb.h>
#include <utmp.h>
#if HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif
#if HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif
#include <termios.h>

#include "config.h"

static int fd= -1;
int crtscts = 1;
static struct termios inittermios;	/* Initial TTY termios */

int setup_line()
{
    if(lock(MODEM_DEVICE)==EOF) {
      fprintf(stderr,"Modem is bussy\n");
      exit(1);
    }
    if ((fd = open("/dev/" MODEM_DEVICE, O_RDWR|O_NDELAY)) < 0) {
	syslog(LOG_ERR, "open(%s): /dev/" MODEM_DEVICE);
	die(1);
    }
    if (fcntl (fd, F_SETFL, O_RDWR) < 0) {
        syslog(LOG_ERR, "fcntl(F_SETFL, ~O_NDELAY): %m");
        die(1);
    }
    /* set line speed, flow control, etc.; clear CLOCAL if modem option */
    set_up_tty(1);
    setdtr(0);
    sleep(1);
    setdtr(1);
    sleep(1);
    set_up_tty(0);
    if (fcntl(fd, F_SETFL, FNDELAY | FASYNC) == -1) {
	syslog(LOG_ERR, "fcntl(F_SETFL, FNDELAY | FASYNC): %m");
	die(1);
    }
    if (ioctl(fd, TIOCEXCL, 0) < 0) {
	syslog (LOG_WARNING, "ioctl(TIOCEXCL): %m");
    }
}


/*
 * cleanup_line - restore anything which needs to be restored before we exit
 */

cleanup_line()
{
    if (fd >= 0) {

	/* drop dtr to hang up */
	setdtr(0);

#if 0
	if (initfdflags != -1 && fcntl(fd, F_SETFL, initfdflags) < 0)
	    syslog(LOG_WARNING, "fcntl(F_SETFL, fdflags): %m");
	initfdflags = -1;
#endif
	if (tcsetattr(fd, TCSAFLUSH, &inittermios) < 0)
	   syslog(LOG_WARNING, "tcsetattr: %m");

	close(fd);
	fd = -1;
    }

    unlock();
}

/*
 * setdtr - control the DTR line on the serial port.
 * This is called from die(), so it shouldn't call die().
 */
setdtr(on)
int on;
{
    int modembits = TIOCM_DTR;

    ioctl(fd, (on? TIOCMBIS: TIOCMBIC), &modembits);
}


/*
 * set_up_tty: Set up the serial port on `fd' for 8 bits, no parity,
 * If `local' is true, set CLOCAL
 * regardless of whether the modem option was specified.
 */
set_up_tty(local)
    int local;
{
    int speed, x;
    struct termios tios;

    if (tcgetattr(fd, &tios) < 0) {
	syslog(LOG_ERR, "tcgetattr: %m");
	die(1);
    }

    inittermios = tios;

#ifdef CRTSCTS
    tios.c_cflag &= ~(CSIZE | CSTOPB | PARENB | CLOCAL | CRTSCTS);
    if (crtscts == 1)
	tios.c_cflag |= CRTSCTS;
#else
    tios.c_cflag &= ~(CSIZE | CSTOPB | PARENB | CLOCAL);
#endif	/* CRTSCTS */

    tios.c_cflag |= CS8 | CREAD | HUPCL;
    if (local)
	tios.c_cflag |= CLOCAL;
    tios.c_iflag = IGNBRK | IGNPAR;
    tios.c_oflag = 0;
    tios.c_lflag = 0;
    tios.c_cc[VMIN] = 1;
    tios.c_cc[VTIME] = 0;

    if (crtscts == 2) {
	tios.c_iflag |= IXOFF;
	tios.c_cc[VSTOP] = 0x13;	/* DC3 = XOFF = ^S */
	tios.c_cc[VSTART] = 0x11;	/* DC1 = XON  = ^Q */
    }

    if (tcsetattr(fd, TCSAFLUSH, &tios) < 0) {
	syslog(LOG_ERR, "tcsetattr: %m");
	die(1);
    }

}

char * make_printable(s,slen)char *s;int slen;
{
  static char b[20001];
  int bp=0,sp;

  for(sp=0;sp<slen;sp++)
    {
      if(s[sp]=='\r')
	{
	  b[bp++]='\\';
	  b[bp++]='r';
	} else
      if(s[sp]=='\n')
	{
	  b[bp++]='\\';
	  b[bp++]='n';
	} else b[bp++]=s[sp];
    }
  b[bp]='\0';
  return b;
}

write_on_line(s)char *s;
{
  int slen=strlen(s);
#if DEBUG
  printf("Write:\t%s\n",make_printable(s,slen));
#endif
  write(fd,s,slen);
}

expect_on_line(s)char *s;
{
  static char b[1000],c[1000];
  char found=0;
  int bp=0,cp=0,blen,slen;
  slen=strlen(s);
#if DEBUG
  printf("Expect:\t%s\n",make_printable(s,slen));
#endif

  bp=0;
  blen=read(fd,b,1000);
  while(bp>=blen)
    {
      usleep(100000);
      bp=0;
      blen=read(fd,b,1000);
    }
  while(bp<blen)
      {
        if(cp==1000)
	  {
	    syslog(LOG_ERR, "To many characters from the modem");
	    die(1000);
	  }
        c[cp++]=b[bp++];
        if(!found && cp>=slen && !strncmp(s,&c[cp-slen],slen))found=1;
        while(bp>=blen && !found)
	  {
	    usleep(100000);
 	    bp=0;
	    blen=read(fd,b,1000);
	  }
      }
#if DEBUG
  printf("Got:\t%s\n",make_printable(c,cp));
#endif
}
